@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU Affero General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * @Author: Hans Zandbelt - hans.zandbelt@zmartzone.eu
+ * @Author: Hans Zandbelt - hans.zandbelt@openidc.com
  *
  **************************************************************************/
 
@@ -138,11 +138,19 @@ bool oauth2_jose_hash_bytes(oauth2_log_t *log, const char *digest,
 	EVP_MD_CTX_init(ctx);
 
 	if ((evp_digest = EVP_get_digestbyname(digest)) == NULL) {
-		oauth2_error(
-		    log,
-		    "no OpenSSL digest algorithm found for algorithm \"%s\"",
-		    digest);
-		goto end;
+		// hack away for el7/x86 where Apache is compiled against
+		// OpenSSL 1.0.2 but NGINX 1.20.1 against OpenSSL 1.1.0
+		if (strcmp(digest, "sha256") == 0) {
+			oauth2_debug(log, "try to directly set EVP_sha256");
+			evp_digest = EVP_sha256();
+		}
+		if (evp_digest == NULL) {
+			oauth2_error(log,
+				     "no OpenSSL digest algorithm found for "
+				     "algorithm \"%s\"",
+				     digest);
+			goto end;
+		}
 	}
 
 	if (!EVP_DigestInit_ex(ctx, evp_digest, NULL))
@@ -854,6 +862,7 @@ _OAUTH2_CFG_CTX_INIT_START(oauth2_jose_jwt_verify_ctx)
 ctx->exp_validate = OAUTH2_CFG_UINT_UNSET;
 ctx->iat_validate = OAUTH2_CFG_UINT_UNSET;
 ctx->iss_validate = OAUTH2_CFG_UINT_UNSET;
+ctx->issuer = NULL;
 ctx->iat_slack_after = OAUTH2_CFG_UINT_UNSET;
 ctx->iat_slack_before = OAUTH2_CFG_UINT_UNSET;
 ctx->jwks_provider = NULL;
@@ -861,14 +870,17 @@ _OAUTH2_CFG_CTX_INIT_END
 
 _OAUTH2_CFG_CTX_CLONE_START(oauth2_jose_jwt_verify_ctx)
 dst->exp_validate = src->exp_validate;
-dst->iat_slack_after = src->iat_slack_after;
-dst->iat_slack_before = src->iat_slack_before;
 dst->iat_validate = src->iat_validate;
 dst->iss_validate = src->iss_validate;
+dst->issuer = oauth2_strdup(src->issuer);
+dst->iat_slack_after = src->iat_slack_after;
+dst->iat_slack_before = src->iat_slack_before;
 dst->jwks_provider = _oauth2_jose_jwks_provider_clone(log, src->jwks_provider);
 _OAUTH2_CFG_CTX_CLONE_END
 
 _OAUTH2_CFG_CTX_FREE_START(oauth2_jose_jwt_verify_ctx)
+if (ctx->issuer)
+	oauth2_mem_free(ctx->issuer);
 if (ctx->jwks_provider)
 	_oauth2_jose_jwks_provider_free(log, ctx->jwks_provider);
 _OAUTH2_CFG_CTX_FREE_END
@@ -1199,14 +1211,15 @@ end:
 static bool
 _oauth2_jose_jwt_payload_validate(oauth2_log_t *log,
 				  oauth2_jose_jwt_verify_ctx_t *jwt_verify_ctx,
-				  const json_t *json_payload, const char *iss)
+				  const json_t *json_payload)
 {
 	bool rc = false;
 
 	oauth2_debug(log, "enter");
 
 	if (_oauth2_jose_jwt_validate_iss(
-		log, json_payload, iss, jwt_verify_ctx->iss_validate) == false)
+		log, json_payload, jwt_verify_ctx->issuer,
+		jwt_verify_ctx->iss_validate) == false)
 		goto end;
 
 	if (_oauth2_jose_jwt_validate_exp(
@@ -1324,8 +1337,8 @@ bool oauth2_jose_jwt_verify(oauth2_log_t *log,
 	}
 
 	if (jwt_verify_ctx) {
-		if (_oauth2_jose_jwt_payload_validate(
-			log, jwt_verify_ctx, *json_payload, NULL) == false) {
+		if (_oauth2_jose_jwt_payload_validate(log, jwt_verify_ctx,
+						      *json_payload) == false) {
 			json_decref(*json_payload);
 			*json_payload = NULL;
 			oauth2_mem_free(*s_payload);
@@ -1641,7 +1654,12 @@ _oauth2_jose_options_jwk_set_rsa_key(oauth2_log_t *log, EVP_PKEY *pkey,
 	rv = _oauth2_jose_verify_options_jwk_add_jwk(log, jwk, params, verify);
 
 end:
-
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+	if (rsa_n)
+		BN_clear_free(rsa_n);
+	if (rsa_e)
+		BN_clear_free(rsa_e);
+#endif
 	if (key_spec.n)
 		oauth2_mem_free(key_spec.n);
 	if (key_spec.e)
